#!/bin/bash
#
# Egyszeru script AwStats konfiguraciok karbantartasara.
# Javasolt naponta egyszer, cron.d vagy cron.daily alol lefuttatni.
# Funkciok:
# * AwStats konfiguracios file-ok letrehozatala listazott, vagy az Apache-ban megadott virtualhostokhoz.
# * Elarvult konfiguracios file-ok felderitese.
#
# 2007-01-03 kovacs.zoltan@mimoza.hu
# Sajat IP kivetele a statisztikakbol (SkipHosts) 
# 2005-05-04 kovacs.zoltan@mimoza.hu
# Elso valtozat
##
                                                                                                                                                
APACHE_DIR="/etc/apache"                        	 # Apache konfiguracio konyvtara
APACHE_LOG_DIR="/var/log/apache"			 # Apache logok konyvtara
APACHE_VHOSTS="$APACHE_DIR/conf.d"              	 # Apache virtualhostok konfiguracios konyvtara
AWK="/usr/bin/awk"                              	 # Az awk hivasa
AWSTATS_CONFIG="/etc/awstats"				 # AWStats konfiguraciok konyvtara
AWSTATS_DATADIR="/var/lib/awstats"      		 # AwStats adattar
AWSTATS_EXC="$APACHE_DIR/awstats.exception"     	 # Kivetelek, soronkent egy csere (opcionalis)
AWSTATS_HOSTS="$APACHE_DIR/awstats.hosts"       	 # Vizsgalando hostok, soronkent egy (opcionalis)
AWSTATS_TEMPLATE="$AWSTATS_CONFIG/awstats.conf.template" # AWStats template
EXCEPTLIST=""                                   	 # space - | delimitalt lista - inicializalas
EXIT_ERR="1"                                    	 # Hibajelzes a hivonak
GREP="/bin/grep"                                	 # A grep hvsa
IFCONFIG="/sbin/ifconfig"				 # Az ifconfig hivasa
NOCHECK="NOSTAT"                                	 # Kizaras jelzese
SED="/bin/sed"                                  	 # A sed hivasa
SERVERLIST=""						 # Space-delimitalt lista - inicializalas
STATS_ROOT_DIR="/var/www/awstats"			 # A statisztikai weblapok alapkonyvtara

# Sajat IP cimek meghatarozasa, ha lehetseges
if [ -x $IFCONFIG ]; then
    SERVERIPS=`echo $($IFCONFIG | $GREP 'inet addr:' | $SED "s/[[:space:]]*inet addr:\([^ ]*\).*/\1/")`
else
    SERVERIPS=""
fi

#
# Fuggvenyek
#

# Ez a fuggveny elokesziti az ellenorzendo virtualhostok listajat.
# A megadott hostok listaja soronkent egy servername-et tartalmaz,
# ures sorok es # karakterrel kezdodo megjegyzes sorok  megengedettek.
# Ha nincsenek megadott hostok, az Apache valamennyi virtualhostja kerul a listaba.
# Az esetleges kiveteleket figyelembe veszi.
# A kivetelek listaja soronkent egy servername-et es egy helyettesitot tartalmaz.
# Ures sorok es # karakterrel kezdodo megjegyzes sorok  megengedettek.
# Ahol a helyettesito $NOCHECK ott a servername nem kerul a listaba.

function make_serverlist {

    local line=""
    local ServerName=""
    
# Elkesziti a kivetelek listajat.

    if [ -r "$AWSTATS_EXC" ] ; then
        EXCEPTLIST=`echo -n $EXCEPTLIST; cat $AWSTATS_EXC | \
                   $AWK '{ if ($0 !~ /^#.*/) print $0 }' | \
                   $AWK 'NF > 0' | while read line
		   do
		       echo "$line |"
		   done`
    fi

# Ha van hostlista, akkor azt dolgozza fel.

    if [ ! -z "`cat $AWSTATS_HOSTS 2>/dev/null | $AWK '{ if ($0 !~ /^#.*/) print $0 }' | $AWK 'NF > 0'`" ] ; then

        SERVERLIST=`cat "$AWSTATS_HOSTS" | \
	            $AWK '{ if ($0 !~ /^#.*/) print $0 }' | \
	            $AWK 'NF > 0' | while read ServerName
                    do
    	        	if [ ! -z "$(subst_exceptlist $ServerName)" ]; then 
			    echo -n "$ServerName "
			fi
		    done`

# Ha nincs hostlista, vegignezi az Apache konfiguraciot, es kiszedi a virtualhostokat.

    else
	SERVERLIST=`$GREP -Rh "ServerName" $APACHE_VHOSTS  | \
	            $GREP -v '#' | $SED -e "s/ServerName//" |while read ServerName
                    do
    	        	if [ ! -z "$(subst_exceptlist $ServerName)" ]; then 
			    echo -n "$ServerName "
			fi
	            done`
    fi
    return
}

# Ez a fuggveny megkeresi az atadott parametert a kivetelek listajaban,
# es ha kell, helyettesit. Az eredmenyt a stdout-ra irja vissza.

function subst_exceptlist {

    local tocheck=$1
    local changed=""
    
# Ha a megadott parameter az  $EXCEPTLIST-ben szerepel, csere kell.

    if [ ! -z "$tocheck" ]; then    
	changed=`echo $EXCEPTLIST | $AWK 'BEGIN { RS = "|" } ; { if ($1 == TOCHECK) print $2 }' TOCHECK=$tocheck`

# Ha volt csere, es az nem $NOCHECK, a cserertek  kerul a kimenetre.

        if [ ! -z "$changed" ] ; then 
	    if [ "$changed" != $NOCHECK ]; then echo "$changed"; fi

# Ha nem kellett cserelni, visszairja a bemenetet.

	else
	    echo "$tocheck"
	fi
    fi
    return
}

#
# Fuggvenyek vege, foprogram.
#

# Vizsgalando virtualhostok osszeallitasa.

make_serverlist

# Hianyzo AwStats konfiguraciok elkeszitese.
# Minden, a $SERVERLIST-ben szereplo virtualhosthoz tartoznia kell
# egy AwStats konfiguracios allomanynak az $AWSTATSDIR-ben is.

for ServerName in $SERVERLIST
do
    script=$0
    logfile=`subst_exceptlist "$ServerName"`
    datetime=`date '+%Y-%m-%d %H:%M'`

    if [ ! -z "$ServerName" -a ! -z "$logfile" ]; then

# Azonositja az access logfile-t - ha nem talalja, hibauzenet keletkezik.

	logfile=$logfile-access.log
        if [ ! -f "$APACHE_LOG_DIR/$logfile" ]; then
	    cat >&1 << EOF
ERROR from $0: Missing $APACHE_LOG_DIR/$logfile! 
When $ServerName's access log isn't $APACHE_LOG_DIR/$logfile, 
write a corresponding line into $AWSTATS_EXC file.

EOF

# A virtualhosthoz letrehoz egy AwStats konfiguracios file-t, ha meg nem letezik.

	else
    	    if [ ! -f "$AWSTATS_CONFIG/awstats.$ServerName.conf" ]; then
		export script ServerName SERVERIPS APACHE_LOG_DIR logfile datetime 
        	cat $AWSTATS_TEMPLATE | envsubst >$AWSTATS_CONFIG/awstats.$ServerName.conf
		cat >&1 << EOF
INFO from $0: New host!
AwStats configuration created for virtualhost $ServerName successfully.

EOF
    	    fi
	fi
    fi
done

# Elarvult AwStats konfiguraciok felderitese.
# Elarvultnak tekintjuk azokat az AwStats konfiguraciokat,
# amelyek leteznek, de nem szerepelnek az aktualis $SERVERLIST-ben.

cd $AWSTATS_CONFIG ; ls -1 awstats.*.conf | \
sed "s/^awstats\.//" | sed "s/.conf\$//" | while read ConfigName
do
    if [ -z "`echo ' '$SERVERLIST' ' | sed -n s/[[:space:]]\*$ConfigName[[:space:]]\*/X/p`" ]; then
	cat >&1 << EOF
WARNING from $0: Orphan configuration!
Virtualhost $ConfigName represented in $AWSTATS_CONFIG 
but isn't found in active virtualhosts' list.
If maintain webstatistics about $ConfigName is unnecessary, 
consider deleting following files also (if any):
$AWSTATS_CONFIG/awstats.$ConfigName.conf
$AWSTATS_DATADIR/awstats??????.$ConfigName.txt	
$STATS_ROOT_DIR/$ConfigName

EOF
    fi
done
