#!/bin/bash
#
# Simple toolkit to manage websites.
# kovacs.zoltan@mimoza.hu (since) 2005-12-10
# Sorry about my terrible English! :-)

# Including headers
HEADERS="/etc/default/website-manager"
. $HEADERS

#
# Functions
#

# wsm_createweb website [email [webmaster [wmpassword [auditor [aupassword]]]]
# Creates a static website with DAV management and web statistics generation.

function wsm_createweb {

# Checking components and circumstances.

    if [ ! $UID -eq 0 ]; then echo -e "$MSG_WSM_ALL_ROOTNEED" >&2; return $EXIT_ERR; fi
    if [ ! -f "$WSMTEMPLATEDIR/$WSMHTTPTEMPLATE" ]; then echo -e "$MSG_WSM_CREATEWEB_NOTEMPLATE $WSMTEMPLATEDIR/$WSMHTTPTEMPLATE" >&2; return $EXIT_ERR; fi
    if [ ! -f "$WSMTEMPLATEDIR/$WSMHTTPSTEMPLATE" ]; then echo -e "$MSG_WSM_CREATEWEB_NOTEMPLATE $WSMTEMPLATEDIR/$WSMHTTPSTEMPLATE" >&2; return $EXIT_ERR; fi
    if [ ! -f "$WSMTEMPLATEDIR/$WSMHTMLPAGE" ]; then echo -e "$MSG_WSM_CREATEWEB_NOTEMPLATE $WSMTEMPLATEDIR/$WSMHTMLPAGE" >&2; return $EXIT_ERR; fi
    if [ ! -f "$WSMTEMPLATEDIR/$WSMSTATPAGE" ]; then echo -e "$MSG_WSM_CREATEWEB_NOTEMPLATE $WSMTEMPLATEDIR/$WSMSTATPAGE" >&2; return $EXIT_ERR; fi
	    
# Website is mandatory and formally checked.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_CREATEWEB_NOWEBSITE" >&2; return $EXIT_ERR; fi

# Webmaster's email is optional, but formally checked.
    local email="$1"; shift
    if [ -z "$email" ]; then email=$DEFAULT_EMAIL; fi
    if [ ! -z "`echo "$email" | $SED "s/$MASKEMAIL//"`" ]; then echo "$MSG_WSM_CREATEWEB_WRONGEMAIL" >&2; return $EXIT_ERR; fi

# Webmaster's username is optional, but formally checked.
    local webmaster="$1"; shift
    if [ -z "$webmaster" ]; then webmaster=$DEFAULT_WEBMASTER; fi
    if [ ! -z "`echo "$webmaster" | $SED "s/$MASKUSER//"`" ]; then echo "$MSG_WSM_CREATEWEB_WRONGWMUSER" >&2; return $EXIT_ERR; fi

# Webmaster's password is optional, but formally checked.
    local wmpassword="$1"; shift
    if [ -z "$wmpassword" ]; then wmpassword=`$MAKEPASSWD`; fi
    if [ ! -z "`echo "$wmpassword" | $SED "s/$MASKPASS//"`" ]; then echo "$MSG_WSM_CREATEWEB_WRONGWMPWD" >&2; return $EXIT_ERR; fi

# Auditor's username is optional, but formally checked.
    local auditor="$1"; shift
    if [ -z "$auditor" ]; then auditor=$DEFAULT_AUDITOR; fi
    if [ ! -z "`echo "$auditor" | $SED "s/$MASKUSER//"`" ]; then echo "$MSG_WSM_CREATEWEB_WRONGAUUSER" >&2; return $EXIT_ERR; fi

# Auditor's password is optional, but formally checked.
    local aupassword="$1"; shift
    if [ -z "$aupassword" ]; then aupassword=`$MAKEPASSWD`; fi
    if [ ! -z "`echo "$aupassword" | $SED "s/$MASKPASS//"`" ]; then echo "$MSG_WSM_CREATEWEB_WRONGAUPWD" >&2; return $EXIT_ERR; fi

# Avoid overwriting any existing content.
    if [ -d "$APACHEDOCROOT/$vhost" ]; then echo "$MSG_WSM_CREATEWEB_DOCROOTEXIST" >&2; return $EXIT_ERR; fi
    if [ -f "$APACHECONFIGDIR/$vhost" ]; then echo "$MSG_WSM_CREATEWEB_HTTPCONFEXIST" >&2; return $EXIT_ERR; fi
    if [ -f "$APACHESSLCONFIGDIR/$vhost" ]; then echo "$MSG_WSM_CREATEWEB_HTTPSCONFEXIST" >&2; return $EXIT_ERR; fi
    if [ -f "$APACHEAUTHDIR/$vhost.user" ]; then echo "$MSG_WSM_CREATEWEB_HTTPUSEREXIST" >&2; return $EXIT_ERR; fi
    if [ -f "$APACHEAUTHDIR/$vhost.group" ]; then echo "$MSG_WSM_CREATEWEB_HTTPGROUPEXIST" >&2; return $EXIT_ERR; fi

# OK, let's go!
    local LOG=""

# Creating web users and groups.
    LOG=`echo -e "$LOG"; wsm_adduser $vhost $webmaster $wmpassword 2>&1`
    LOG=`echo -e "$LOG"; wsm_addgroup $vhost "admin" 2>&1`
    LOG=`echo -e "$LOG"; wsm_addusertogroup $vhost $webmaster "admin" 2>&1`
    LOG=`echo -e "$LOG"; wsm_adduser $vhost $auditor $aupassword 2>&1`
    LOG=`echo -e "$LOG"; wsm_addgroup $vhost "stats" 2>&1`
    LOG=`echo -e "$LOG"; wsm_addusertogroup $vhost $auditor "stats" 2>&1`
    
# Creating web document root and minimal content: 
# indexpage, download, binary, config, download, upload directory and stats link.
    if [ ! -d $APACHEDOCROOT/$vhost ]; then
	mkdir -m 2750 $APACHEDOCROOT/$vhost
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost $MSG_WSM_CREATEWEB_CREATED."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost $MSG_WSM_CREATEWEB_EXISTS."`
    fi
		
    if [ ! -f $APACHEDOCROOT/$vhost/$DEFAULT_PAGE ]; then
        export vhost
	cat $WSMTEMPLATEDIR/$WSMHTMLPAGE | $ENVSUBST > $APACHEDOCROOT/$vhost/$DEFAULT_PAGE
	chmod 640 $APACHEDOCROOT/$vhost/$DEFAULT_PAGE
	chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$DEFAULT_PAGE
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$DEFAULT_PAGE $MSG_WSM_CREATEWEB_CREATED."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$DEFAULT_PAGE $MSG_WSM_CREATEWEB_EXISTS."`
    fi

    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEDNLOADDIR ]; then
	mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEDNLOADDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEDNLOADDIR
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEDNLOADDIR $MSG_WSM_CREATEWEB_CREATED."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEDNLOADDIR $MSG_WSM_CREATEWEB_EXISTS."`
    fi

    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR ]; then
	mkdir -m 2770 $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR
	if [ -x $SETFACL ]; then
	    $SETFACL -d -m u:$APACHEADMIN:rwX $APACHEDOCROOT/$vhost/$APACHEUPLOADDIR
	fi
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEUPLOADDIR $MSG_WSM_CREATEWEB_CREATED."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEUPLOADDIR $MSG_WSM_CREATEWEB_EXISTS."`
    fi

    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEVHBINDIR ]; then
	mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEVHBINDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHBINDIR
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEVHBINDIR $MSG_WSM_CREATEWEB_CREATED."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEVHBINDIR $MSG_WSM_CREATEWEB_EXISTS."`
    fi

    if [ ! -d $APACHEDOCROOT/$vhost/$APACHEVHCONFDIR ]; then
	mkdir -m 2750 $APACHEDOCROOT/$vhost/$APACHEVHCONFDIR
        chown $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$APACHEVHCONFDIR
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEVHCONFDIR $MSG_WSM_CREATEWEB_CREATED."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$APACHEVHCONFDIR $MSG_WSM_CREATEWEB_EXISTS."`
    fi

    if [ ! -L $APACHEDOCROOT/$vhost/$AWSTATSLINK ]; then
	ln -s $AWSTATSWEBROOT/$vhost $APACHEDOCROOT/$vhost/$AWSTATSLINK
        chown -h $APACHEADMIN:$APACHEGROUP $APACHEDOCROOT/$vhost/$AWSTATSLINK
	LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$AWSTATSLINK -> $AWSTATSWEBROOT/$vhost symlinked."`
    else
        LOG=`echo -e "$LOG\n$APACHEDOCROOT/$vhost/$AWSTATSLINK $MSG_WSM_CREATEWEB_EXISTS."`
    fi

# Apache webserver configuration.
    if [ ! -f $APACHECONFIGDIR/$vhost ]; then
	DATE=`date '+%Y-%m-%d %H:%M'`
	export APACHEDOCROOT APACHELOGDIR APACHEVHBINDIR APACHEVHCONFDIR APACHEDNLOADDIR APACHEUPLOADDIR APACHEWEBIP APACHEWEBPORT DATE PHPLOGDIR PHP4LOGDIR PHP5LOGDIR email vhost
    	cat $WSMTEMPLATEDIR/$WSMHTTPTEMPLATE | $ENVSUBST > $APACHECONFIGDIR/$vhost
	chown root:root $APACHECONFIGDIR/$vhost
        chmod 644 $APACHECONFIGDIR/$vhost
        LOG=`echo -e "$LOG\n$APACHECONFIGDIR/$vhost $MSG_WSM_CREATEWEB_CREATED."`
        LOG=`echo -e "$LOG"; $APACHEDAEMON "reload" 2>&1`
    else
	LOG=`echo -e "$LOG\n$APACHECONFIGDIR/$vhost $MSG_WSM_CREATEWEB_EXISTS."`
    fi

# Apache SSL webserver configuration.
    if [ ! -f $APACHESSLCONFIGDIR/$vhost ]; then
	DATE=`date '+%Y-%m-%d %H:%M'`
	export APACHEAUTHDIR APACHEDOCROOT APACHELOGDIR APACHESSLIP APACHESSLPORT APACHEVHBINDIR APACHEVHCONFDIR APACHEDNLOADDIR APACHEUPLOADDIR DATE PHPLOGDIR PHP4LOGDIR PHP5LOGDIR email vhost
    	cat $WSMTEMPLATEDIR/$WSMHTTPSTEMPLATE | $ENVSUBST > $APACHESSLCONFIGDIR/$vhost
	chown root:root $APACHESSLCONFIGDIR/$vhost
        chmod 644 $APACHESSLCONFIGDIR/$vhost
        LOG=`echo -e "$LOG\n$APACHESSLCONFIGDIR/$vhost $MSG_WSM_CREATEWEB_CREATED."`
        LOG=`echo -e "$LOG"; $APACHESSLDAEMON "reload" 2>&1`
    else
	LOG=`echo -e "$LOG\n$APACHESSLCONFIGDIR/$vhost $MSG_WSM_CREATEWEB_EXISTS."`
    fi

# That's all, folks :-)
    echo -e "$LOG"
    return $EXIT_SUCC
}

# wsm_saveweb website [--noapache] [--noawstats] [--nobins] [--nodocs] [--nologs]
# Save a website - webpages, users, logs, configurations, statistics.

function wsm_saveweb {

# Website is mandatory
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# Parsing optional options
    local saveapache="Y"
    local saveawstats="Y"
    local savebins="Y"
    local savedocs="Y"
    local savelogs="Y"
    local option="$1"; shift
    while [ ${#option} -gt 0 ];
    do
	case $option in
	    --noapache)
		saveapache="N"
		;;
	    --noawstats)
		saveawstats="N"
		;;
	    --nobins)
		savebins="N"
		;;
	    --nodocs)
		savedocs="N"
		;;
	    --nologs)
		savelogs="N"
		;;
	esac
	option="$1"; shift
    done

    local backupfile=""
    local datetime="`date '+%Y%m%d%H%M%S'`"
    local taroptions=""

# Web content from docroot save
    if [ $savedocs = "Y" ]; then
	backupfile="`pwd`/`hostname`.$vhost.$datetime.docroot.tar.gz"
	if [ -d $APACHEDOCROOT/$vhost ]; then
	    ( cd  $APACHEDOCROOT/$vhost ;
	      $TAR 'czf' $backupfile *
	    )
	    wait
	else
	    echo "$MSG_WSM_SAVEWEB_NODOCROOT $APACHEDOCROOT/$vhost" >&2
	fi
    fi

# Binaries from binary path save
    if [ $savebins = "Y" ]; then
	backupfile="`pwd`/`hostname`.$vhost.$datetime.binary.tar.gz"
	if [ -d $APACHEBINROOT/$vhost ]; then
	    ( cd  $APACHEBINROOT/$vhost ;
	      $TAR 'czf' $backupfile *
	    )
	    wait
	else
	    echo "$MSG_WSM_SAVEWEB_NOBINROOT $APACHEBINROOT/$vhost" >&2
	fi
    fi

# Apache configuration, users and groups
    if [ $saveapache = "Y" ]; then
        backupfile="`pwd`/`hostname`.$vhost.$datetime.apache.tar"
	if [ -f "$APACHECONFIGDIR/$vhost" ]; then
    	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
    	    $TAR $taroptions'f' $backupfile $APACHECONFIGDIR/$vhost 2>/dev/null
	else
    	    echo "$MSG_WSM_SAVEWEB_NOAPACHE $APACHECONFIGDIR/$vhost" >&2
	fi
	if [ -f "$APACHESSLCONFIGDIR/$vhost" ]; then
    	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
    	    $TAR $taroptions'f' $backupfile $APACHESSLCONFIGDIR/$vhost 2>/dev/null
	else
    	    echo "$MSG_WSM_SAVEWEB_NOAPACHE $APACHESSLCONFIGDIR/$vhost" >&2
	fi
	if [ -f "$APACHEAUTHDIR/$vhost.group" ]; then
	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
	    $TAR $taroptions'f' $backupfile $APACHEAUTHDIR/$vhost.group 2>/dev/null
	else
	    echo "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.group" >&2
	fi
	if [ -f "$APACHEAUTHDIR/$vhost.user" ]; then
	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
	    $TAR $taroptions'f' $backupfile $APACHEAUTHDIR/$vhost.user 2>/dev/null
	else
	    echo "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.user" >&2
	fi
	if [ -f $backupfile ]; then $GZIP $backupfile; fi
    fi

# Logs
    if [ $savelogs = "Y" ]; then
	backupfile="`pwd`/`hostname`.$vhost.$datetime.apachelogs.tar.gz"
# Apache logs
	( cd $APACHELOGDIR ;
          if [ ! -z "`ls $vhost* 2>/dev/null`" ]; then
	      $TAR 'czf' $backupfile $vhost*
    	  else
	      echo -e "$MSG_WSM_SAVEWEB_NOAPACHELOG $APACHELOGDIR/$vhost*" >&2
    	  fi
	)
	wait
# Optional PHP logs
	backupfile="`pwd`/`hostname`.$vhost.$datetime.phplogs.tar.gz"
	if [ -d "$PHPLOGDIR" ]; then
	    ( cd $PHPLOGDIR ;
              if [ ! -z "`ls $vhost* 2>/dev/null`" ]; then
	          $TAR 'czf' $backupfile $vhost*
    	      else
	          echo -e "$MSG_WSM_SAVEWEB_NOPHPLOG $PHPLOGDIR/$vhost*" >&2
    	      fi
	    )
	fi
	wait
    fi

# AwStats configuration, results and webpages
    if [ $saveawstats = "Y" ]; then
	backupfile="`pwd`/`hostname`.$vhost.$datetime.awstats.tar"
	if [ -f "$AWSTATSCONFIGDIR/awstats.$vhost.conf" ]; then
	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
	    $TAR $taroptions'f' $backupfile $AWSTATSCONFIGDIR/awstats.$vhost.conf 2>/dev/null
	else
	    echo "$MSG_WSM_SAVEWEB_NOAWSTATS $AWSTATSCONFIGDIR/awstats.$vhost.conf" >&2
	fi
	if [ ! -z "`ls $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null`" ]; then
	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
	    $TAR $taroptions'f' $backupfile $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null
	else
	    echo "$MSG_WSM_SAVEWEB_NOAWSTATS $AWSTATSCONTDIR/*.$vhost.*" >&2
	fi
	if [ -d "$AWSTATSWEBROOT/$vhost" ]; then
	    if [ ! -f $backupfile ]; then taroptions="c"; else taroptions="r"; fi
	    $TAR $taroptions'f' $backupfile $AWSTATSWEBROOT/$vhost 2>/dev/null
	else
	    echo "$MSG_WSM_SAVEWEB_NOAWSTATS $AWSTATSWEBROOT/$vhost/*" >&2
	fi
	if [ -f $backupfile ]; then $GZIP $backupfile; fi
    fi
}

# wsm_removeweb website [force]
# Remove a website - webpages, binaries, users, logs, configurations, statistics.

function wsm_removeweb {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# Confirmation request if not forced.
    if [ "$1" != "force" ]; then
	read -s -n1 -p "$MSG_WSM_RMVWEB_CONFIRM"; echo
	if [ "$REPLY" != "y" -a "$REPLY" != "Y" ]; then return $EXIT_ERR; fi
    fi

# Removing Apache configuration, users and groups.
    if [ -f "$APACHECONFIGDIR/$vhost" ]; then
        rm -f $APACHECONFIGDIR/$vhost 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $APACHECONFIGDIR/$vhost"
    else
        echo "$MSG_WSM_RMVWEB_NOAPACHE $APACHECONFIGDIR/$vhost" >&2
    fi
    if [ -f "$APACHESSLCONFIGDIR/$vhost" ]; then
        rm -f $APACHESSLCONFIGDIR/$vhost 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $APACHESSLCONFIGDIR/$vhost"
    else
        echo "$MSG_WSM_RMVWEB_NOAPACHE $APACHESSLCONFIGDIR/$vhost" >&2
    fi
    if [ -f "$APACHEAUTHDIR/$vhost.group" ]; then
        rm -f $APACHEAUTHDIR/$vhost.group 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $APACHEAUTHDIR/$vhost.group"
    else
	echo "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.group" >&2
    fi
    if [ -f "$APACHEAUTHDIR/$vhost.user" ]; then
        rm -f $APACHEAUTHDIR/$vhost.user 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $APACHEAUTHDIR/$vhost.user"
    else
	echo "$MSG_WSM_SAVEWEB_NOAPACHE $APACHEAUTHDIR/$vhost.user" >&2
    fi

# Removing Apache logs.
    if [ ! -z "`ls $APACHELOGDIR/$vhost* 2>/dev/null`" ]; then
	rm -f $APACHELOGDIR/$vhost* 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $APACHELOGDIR/$vhost*"
    else
	echo -e "$MSG_WSM_RMVWEB_NOAPACHELOG $APACHELOGDIR/$vhost*" >&2
    fi

# Removing PHP logs.
    if [ ! -z "`ls $PHPLOGDIR/$vhost* 2>/dev/null`" ]; then
	rm -f $PHPLOGDIR/$vhost* 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $PHPLOGDIR/$vhost*"
    else
	echo -e "$MSG_WSM_RMVWEB_NOPHPLOG $PHPLOGDIR/$vhost*" >&2
    fi

# Removing AwStats configuration, results and webpages
    if [ -f "$AWSTATSCONFIGDIR/awstats.$vhost.conf" ]; then
	rm -f $AWSTATSCONFIGDIR/awstats.$vhost.conf 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $AWSTATSCONFIGDIR/awstats.$vhost.conf"
    else
	echo "$MSG_WSM_RMVWEB_NOAWSTATS $AWSTATSCONFIGDIR/awstats.$vhost.conf" >&2
    fi
    if [ ! -z "`ls $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null`" ]; then
	rm -f $AWSTATSCONTDIR/*.$vhost.* 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $AWSTATSCONTDIR/*.$vhost.*"
    else
	echo "$MSG_WSM_RMVWEB_NOAWSTATS $AWSTATSCONTDIR/*.$vhost.*" >&2
    fi
    if [ -d "$AWSTATSWEBROOT/$vhost" ]; then
	rm -Rf $AWSTATSWEBROOT/$vhost 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $AWSTATSWEBROOT/$vhost"
    else
	echo "$MSG_WSM_RMVWEB_NOAWSTATS $AWSTATSWEBROOT/$vhost/*" >&2
    fi

# Remove web content from docroot.
    if [ -d $APACHEDOCROOT/$vhost ]; then
	rm -Rf $APACHEDOCROOT/$vhost 2>/dev/null
        echo "$MSG_WSM_RMVWEB_REMOVED $APACHEDOCROOT/$vhost"
    else
	echo "$MSG_WSM_RMVWEB_NODOCROOT $APACHEDOCROOT/$vhost" >&2
    fi

# Remove binaries from binary store.
    if [ -d $APACHEBINROOT/$vhost ]; then
	rm -Rf $APACHEBINROOT/$vhost 2>/dev/null
	if [ ! -d $APACHEBINROOT/$vhost ]; then
    	    echo "$MSG_WSM_RMVWEB_REMOVED $APACHEBINROOT/$vhost"
	else
	    echo "$MSG_WSM_RMVWEB_NOTREMOVED $APACHEBINROOT/$vhost" >&2
	fi
    else
	echo "$MSG_WSM_RMVWEB_NOBINROOT $APACHEBINROOT/$vhost" >&2
    fi

# Reloading webservers' configuration.
    $APACHEDAEMON "reload" 
    $APACHESSLDAEMON "reload" 

    return $EXIT_SUCC
}

# wsm_adduser website username [password]
# Adds a user to website's htpasswd file. If already exist, changes password.
# Finally echoes website, username and password used.

function wsm_adduser {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# User name is mandatory and formally checked.
    local username="$1"; shift
    if [ -z "$username" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi
    if [ ! -z "`echo "$username" | $SED "s/$MASKUSER//"`" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi

# Password is optional, if empty, random password used.
# Password is formally checked also.
    local password="$1"; shift
    if [ -z "$password" ]; then password=`$MAKEPASSWD`; fi
    if [ ! -z "`echo "$password" | $SED "s/$MASKPASS//"`" ]; then echo "$MSG_WSM_ADDUSER_WRONGPWD" >&2; return $EXIT_ERR; fi

# Creates user file, if it's not found, then calls htpasswd.
    $TOUCH $APACHEAUTHDIR/$vhost.user
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.user
    chmod 440 $APACHEAUTHDIR/$vhost.user
    $HTPASSWD -b $APACHEAUTHDIR/$vhost.user $username $password 2>/dev/null
    echo $vhost $username $password
    return $EXIT_SUCC
}

# wsm_deluser website username
# Deletes given user from website's htpasswd file.
# Deletes group memeberships also, using 
# wsm_listusersgroup and wsm_deluserfromgroup functions.

function wsm_deluser {

# Website is mandatory and must exist.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi
    if [ ! -f "$APACHEAUTHDIR/$vhost.user" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi

# Removing group memberships.
    local membership=`wsm_listusersgroup $vhost $username`
    local groupname=""
    for groupname in $membership
    do
	wsm_deluserfromgroup $vhost $username $groupname
    done

# Calls htpasswd with -D parameter.
    $HTPASSWD -D $APACHEAUTHDIR/$vhost.user $username 2>/dev/null
    return $EXIT_SUCC
}

# wsm_addgroup website groupname
# Adds a user's group for website.

function wsm_addgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# Group name is mandatory and formally checked.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi
    if [ ! -z "`echo "$groupname" | $SED "s/$MASKGROUP//"`" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi

# Creates group file and group line, if it's not found.
    $TOUCH $APACHEAUTHDIR/$vhost.group
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then 
	echo "$groupname:" >> $APACHEAUTHDIR/$vhost.group
    fi
    return $EXIT_SUCC
}

# wsm_delgroup website groupname
# Delete a group line from website's group file.

function wsm_delgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# Group name is mandatory and must exist.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi

# Delete group line.
    mv $APACHEAUTHDIR/$vhost.group $APACHEAUTHDIR/$vhost.group.bak
    cat $APACHEAUTHDIR/$vhost.group.bak | $GREP -v "$groupname:" >> $APACHEAUTHDIR/$vhost.group
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    rm $APACHEAUTHDIR/$vhost.group.bak
    return $EXIT_SUCC
}

# wsm_listusersgroup website username
# Lists all groups containing given user.

function wsm_listusersgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi

# Enumerates group lines and finds username.
    cat $APACHEAUTHDIR/$vhost.group 2>/dev/null | \
    $SED "s/\(:\|,\)/ /g" | \
    $SED -n "s/^\($MASKGROUP\)\(.*\)\($username\)\(.*\)$/\1/p"
    return $EXIT_SUCC
}

# wsm_addusertogroup website user group
# Adds a user to a group of website.

function wsm_addusertogroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi

# Group name is mandatory and must exist.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi

# Gets group line then add user if isn't found.
    mv $APACHEAUTHDIR/$vhost.group $APACHEAUTHDIR/$vhost.group.bak
    cat $APACHEAUTHDIR/$vhost.group.bak | while read line
    do
	if [ ! -z "`echo $line | $SED -n "s/^\($groupname:\)\(.*\)$/\1/p"`" ]; then
	    if [ -z "`echo $line | $SED -n "s/^\($groupname\)\(.*\)\(:\|,\)$username\(,\|$\)\(.*\)$/\1/p"`" ]; then
		if [ -z "`echo $line | $SED -n "s/^\(.*\):$/\1/p"`" ]; then
		    echo "$line,$username" >> $APACHEAUTHDIR/$vhost.group
		else
		    echo "$line$username" >> $APACHEAUTHDIR/$vhost.group
		fi
	    else
		echo $line >> $APACHEAUTHDIR/$vhost.group
	    fi
	else
	    echo $line >> $APACHEAUTHDIR/$vhost.group
	fi
    done
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    rm $APACHEAUTHDIR/$vhost.group.bak
    return $EXIT_SUCC
}

# wsm_deluserfromgroup website user group
# Delete given website user from the given group.

function wsm_deluserfromgroup {

# Website is mandatory.
    local vhost="$1"; shift
    if [ -z "$vhost" ]; then echo "$MSG_WSM_ALL_NOWEBSITE" >&2; return $EXIT_ERR; fi

# User name is mandatory and must exist.
    local username="$1"; shift
    if [ -z "$username" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.user | $SED -n "s/$username:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOUSER" >&2; return $EXIT_ERR; fi

# Group name is mandatory and must exist.
    local groupname="$1"; shift
    if [ -z "$groupname" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi
    if [ -z "`cat $APACHEAUTHDIR/$vhost.group | $SED -n "s/$groupname:/X/p"`" ]; then echo "$MSG_WSM_ALL_NOGROUP" >&2; return $EXIT_ERR; fi

# Gets group lines then removes user if exist.
    mv $APACHEAUTHDIR/$vhost.group $APACHEAUTHDIR/$vhost.group.bak
    cat $APACHEAUTHDIR/$vhost.group.bak | while read line
    do
	if [ ! -z "`echo $line | $SED -n "s/^\($groupname:\)\(.*\)$/\1/p"`" ]; then
	    echo $line | $SED "s/\(,\|:\)$username\(,\|$\)/\1\2/" | \
	         $SED "s/:,/:/" | $SED "s/,,/,/" | $SED "s/,$//" >> $APACHEAUTHDIR/$vhost.group
	else
	    echo $line >> $APACHEAUTHDIR/$vhost.group
	fi
    done
    chown $APACHEADMIN:$APACHEUSER $APACHEAUTHDIR/$vhost.group
    chmod 440 $APACHEAUTHDIR/$vhost.group
    rm $APACHEAUTHDIR/$vhost.group.bak
    return $EXIT_SUCC
}

#
# Main program - handling options
#

option=`echo $1 | $SED -n "s/^-\+//p"`; if [ ! -z "$option" ]; then shift; fi
case "$option" in
    "cw"|"createweb")
	wsm_createweb $*; exit $?
    ;;
    "rw"|"removeweb")
	wsm_removeweb $*; exit $?
    ;;
    "sw"|"saveweb")
	wsm_saveweb $*; exit $?
    ;;
    "au"|"adduser")
	wsm_adduser $*; exit $?
    ;;
    "du"|"deluser")
	wsm_deluser $*; exit $?
    ;;
    "ag"|"addgroup")
	wsm_addgroup $*; exit $?
    ;;
    "dg"|"delgroup")
	wsm_delgroup $*; exit $?
    ;;
    "aug"|"addusertogroup")
	wsm_addusertogroup $*; exit $?
    ;;
    "dug"|"deluserfromgroup")
	wsm_deluserfromgroup $*; exit $?
    ;;
    "lug"|"listusersgroup")
	wsm_listusersgroup $*; exit $?
    ;;
    *)
	echo -e "$USAGE"; exit $EXIT_ERR
    ;;
esac
